from ragloader.conf.config import Config
from ragloader.files_indexing import Document, File
from ragloader.documents_parsing.common.file_parser import BaseFileParser
from ragloader.documents_parsing.common.parsed_items import ParsedFile, ParsedDocument
from ragloader.documents_parsing.parsers import FileParsers


class DocumentParser:
    """This class is used to parse a document (extract its content)."""

    def __init__(self, config: Config):
        self.extensions_parsers: dict = config["pipeline_stages"]["documents_parsing"]["parsers"]

    def parse(self, document: Document) -> ParsedDocument | None:
        """
        This method converts a `Document` object to a `ParsedDocument` object.
        It iterates over all files in the document and combines all `ParsedFile` objects together.
        """
        files: list[File] = document.files

        parsed_document: ParsedDocument = ParsedDocument(document)

        for file in files:
            parser_name: str = self.extensions_parsers.get(file.extension)
            if parser_name is None:
                raise Exception(f"File extension not supported: {file.extension}")

            if parser_name == "auto":
                raise NotImplementedError(f"Parser {parser_name} not implemented.")

            parser_class: type = FileParsers.__getitem__(parser_name).value
            parser: BaseFileParser = parser_class()

            parsed_file: ParsedFile = parser.parse(file)
            parsed_document.add_parsed_file(parsed_file)

            print("Parsed file:", file.file_path)

        return parsed_document
