from ragloader.conf.config import Config
from ragloader.documents_classification.classified_items import ParsedDocumentWithContent
from ragloader.documents_splitting.common.chunked_items import ChunkedDocument
from ragloader.documents_splitting.common.text_splitter import BaseTextSplitter
from ragloader.documents_splitting.splitters import TextSplitters


class DocumentSplitter:
    """Class for splitting a document into chunks."""

    def __init__(self, config: Config):
        self.default_splitters: dict = config["pipeline_stages"]["documents_splitting"]["splitters"]
        self.splitters_params: dict = config["pipeline_stages"]["documents_splitting"]["splitters_params"]

    def split(self, document: ParsedDocumentWithContent) -> ChunkedDocument:
        """Splits a document into chunks based on its class and schema."""
        chunked_document: ChunkedDocument = ChunkedDocument(document)
        splitter_name: str | None = self.default_splitters.get(document.document_class)

        if not splitter_name:
            raise ValueError(f"No default splitter was specified for the class {document.document_class}")

        splitter_params: dict = self.splitters_params.get(splitter_name, {})
        try:
            splitter_class: type = TextSplitters.__getitem__(splitter_name).value
        except KeyError:
            raise NotImplementedError(f"Splitter {splitter_name} is not yet implemented")
        splitter: BaseTextSplitter = splitter_class(**splitter_params)

        chunks: list[str] = splitter.split(document.document_content, document.document_schema)
        for chunk in chunks:
            chunked_document.add_chunk(chunk)

        return chunked_document
