from __future__ import annotations
from uuid import UUID, uuid4
from pathlib import Path


class File:
    """Represents a file to be parsed."""

    def __init__(self, file_path: Path | str):
        self.uuid: UUID = uuid4()

        self.file_path: Path = Path(file_path)
        self.file_name: str = self.file_path.name
        self.extension: str = self.file_path.suffix
        self.metadata: dict | None = None


    def __repr__(self):
        return f"File(name={self.file_name})"


class Document:
    """Represents a document which consists of a list of files."""

    def __init__(self, document_path: Path | str):
        self.uuid: UUID = uuid4()

        self.document_path: Path = Path(document_path)
        self.document_name: str = self.document_path.name
        self.files: list[File] = []

    def add_file(self, file: File):
        self.files.append(file)

    @classmethod
    def from_file_path(cls, file_path: Path):
        """
        Create a Document object based on a single file.

        Args:
            file_path (Path): path to the file
        """
        document: Document = cls(document_path=file_path)
        file: File = File(file_path)
        document.add_file(file)
        return document

    @classmethod
    def from_directory_path(cls, directory_path: Path):
        """
        Create a Document object based on a directory with files.

        Args:
            directory_path (Path): path to the directory with files
        """
        document: Document = cls(document_path=directory_path)
        for file_path in sorted(filter(Path.is_file, directory_path.glob("*"))):
            document.add_file(File(file_path))

        return document if document.files else None

    def __repr__(self):
        files_repr: str = ", ".join(repr(file) for file in self.files)
        return f"Document(n_files={len(self.files)}, files=[\n\t\t\t\t{files_repr}\n\t\t\t])"


class Group:
    """Represents a group of documents."""

    def __init__(self, group_path: Path | str):
        self.uuid: UUID = uuid4()

        self.group_path: Path = Path(group_path)
        self.group_name: str = self.group_path.name
        self.documents: list[Document] = []


    def add_document(self, document: Document):
        self.documents.append(document)

    def __repr__(self):
        documents_repr: str = ",\n\t\t".join(repr(document) for document in self.documents)
        return (f"Group(n_docs={len(self.documents)}, name='{self.group_name}',"
                f" documents=[\n\t\t{documents_repr}\n\t])")


class DocumentsStructure:
    """Represents the whole structure of documents to be parsed."""

    def __init__(self, path: Path | str):
        self.uuid: UUID = uuid4()
        self.root_path: Path = Path(path)
        self.name: str = self.root_path.name
        self.groups: list[Group] = []

    def add_group(self, group: Group):
        self.groups.append(group)

    def to_json(self):
        return {
            "uuid": str(self.uuid),
            "root_path": str(self.root_path),
            "name": self.name,
            "groups": [
                {
                    "uuid": str(group.uuid),
                    "group_path": str(group.group_path),
                    "group_name": group.group_name,
                    "documents": [
                        {
                            "uuid": str(document.uuid),
                            "document_name": document.document_name,
                            "files": [
                                {
                                    "uuid": str(file.uuid),
                                    "file_npath": file.file_path,
                                    "file_name": file.file_name,
                                    "extension": file.extension,
                                    "metadata": file.metadata
                                }
                                for file in document.files
                            ],
                        }
                        for document in group.documents
                    ]
                }
                for group in self.groups
            ]
        }

    def __repr__(self):
        separator_length: int = 30
        group_separator: str = separator_length * "="
        groups_repr: str = f"\n\t{group_separator}\n\t".join(repr(group) for group in self.groups)
        return (f"DocumentsStructure(n_groups={len(self.groups)}, name='{self.name}',"
                f" groups=[\n\t{group_separator}\n\t{groups_repr}\n\t{group_separator}\n])")
