from pathlib import Path
from qdrant_client.models import PointStruct
from concurrent.futures import ThreadPoolExecutor
import threading
import random
import os

from ragloader.chunks_embedding.common.document_embedder import DocumentEmbedder
from ragloader.conf.config import Config
from ragloader.db.qdrant_connector import QdrantConnector
from ragloader.documents_parsing import DocumentParser, ParsedDocument
from ragloader.documents_classification import ParsedDocumentWithContent
from ragloader.files_indexing import DocumentsStructure, FilesIndexer, Document
from ragloader.documents_splitting import ChunkedDocument, DocumentSplitter


class UploadOrchestrator:
    def __init__(self,
                 data_directory: Path | str,
                 config: Config | Path | str | None = None):
        if config is None:
            self.config: Config = Config()
        elif isinstance(config, (str, Path)):
            self.config: Config = Config(Path(config))
        elif isinstance(config, Config):
            self.config: Config = config
        else:
            raise TypeError(f"Invalid type for config: {type(config)}")

        self.data_directory: Path = Path(data_directory)
        self.documents_structure: DocumentsStructure | None = None

        self.lock = threading.Lock()
        self.qdrant: QdrantConnector = QdrantConnector(self.config["db"]["qdrant"])

        self.document_parser: DocumentParser = DocumentParser(self.config)
        self.document_splitter: DocumentSplitter = DocumentSplitter(self.config)
        self.document_embedder: DocumentEmbedder = DocumentEmbedder(self.config)

        self.stages: list[str] = list(self.config["pipeline_stages"].keys())
        self.collection_names: dict[str, str] = dict().fromkeys(self.stages)
        self.initialize_collections()

    def initialize_collections(self):
        collections_names_base = ["parsed_files", "classified_documents", "chunked_documents"]
        for base_name, stage in zip(collections_names_base, self.stages):
            collection_name = f"{self.config['pipeline_stages'][stage]['label']}__{base_name}"
            self.collection_names[stage] = collection_name
            self.qdrant.create_collection(collection_name, if_exists="ignore")

        embedding_label = self.config['pipeline_stages']['documents_embedding']['label']
        embeddings_collection_name = f"{embedding_label}__embedded_chunks"
        self.collection_names["documents_embedding"] = embeddings_collection_name
        self.qdrant.create_collection(
            embeddings_collection_name,
            vectors_length=self.document_embedder.embedding_model.vector_length,
            if_exists="ignore")

    def upload(self):
        self.index_files()
        documents: list[Document] = [document
                                     for group in self.documents_structure.groups
                                     for document in group.documents]

        with ThreadPoolExecutor(max_workers=4) as executor:
            futures = [executor.submit(self.pipeline, document) for document in documents]
            for future in futures:
                future.result()

    def pipeline(self, document: Document):
        parsed_document: ParsedDocument = self.parse_document(document)
        classified_document: ParsedDocumentWithContent = self.classify_document(parsed_document)
        extracted_document: ParsedDocumentWithContent = self.extract_content(classified_document)
        chunked_document: ChunkedDocument = self.split_document(extracted_document)
        self.embed_chunked_document(chunked_document)

    def index_files(self):
        indexer = FilesIndexer(self.data_directory)
        self.documents_structure = indexer.scan()

    def parse_document(self, document: Document) -> ParsedDocument:
        parsed_document: ParsedDocument = self.document_parser.parse(document)

        if parsed_document is not None:
            for file_idx, parsed_file in enumerate(parsed_document.parsed_files):
                payload: dict = {
                    "file_path": parsed_file.file_path,
                    "file_name": parsed_file.file_name,
                    "file_extension": parsed_file.file_extension,
                    "file_metadata": parsed_file.file_metadata,
                    "file_content": parsed_file.file_content,
                }
                point: PointStruct = PointStruct(
                    id=str(parsed_file.file_uuid), vector={}, payload=payload)
                with self.lock:
                    self.qdrant.add_record(self.collection_names["documents_parsing"], point)

        return parsed_document

    def classify_document(self, parsed_document: ParsedDocument) -> ParsedDocumentWithContent:
        # classified_document: ParsedDocumentWithContent = ParsedDocumentWithContent(parsed_document)
        # TODO: implement classification logic and decide whether to add all as separate collection or not
        # as only the document class will differ
        document_class: str = "unstructured_text"
        parsed_document_with_class = ParsedDocumentWithContent(parsed_document)
        parsed_document_with_class.add_document_class(document_class)
        # classified_group.documents_with_content.append(parsed_document_with_class)

        return parsed_document_with_class

    def extract_content(self, classified_document: ParsedDocumentWithContent) -> ParsedDocumentWithContent:
        if classified_document.document_class == "unstructured_text":
            classified_document.add_document_schema({})
        else:
            # TODO: implement content / schema extraction logic
            pass
        # Add the documents with content to separate collection

        return classified_document

    def split_document(self, extracted_document: ParsedDocumentWithContent) -> ChunkedDocument:
        """
        Splits a group of parsed documents (with a class and a schema) into chunked documents.
        Chunked documents are added to the database along with the original document information.
        Chunks of a given document are stored in a list of dictionaries, where each dictionary contains:
        - chunk_index: index of the chunk in the document
        - content: text content of the chunk
        - metadata: metadata of the chunk
        """
        chunked_document: ChunkedDocument = self.document_splitter.split(extracted_document)

        chunks_data: list = [
            {"chunk_index": chunk.chunk_index, "content": chunk.content,
             "metadata": chunk.document_metadata}
            for chunk in chunked_document.chunks
        ]
        payload: dict = {
            "document_name": chunked_document.document_name,
            "document_content": chunked_document.document_content,
            "document_class": chunked_document.document_class,
            "document_schema": chunked_document.document_schema,
            "document_chunks": chunks_data,
        }
        point: PointStruct = PointStruct(id=str(chunked_document.document_uuid), vector={},
                                         payload=payload)
        with self.lock:
            self.qdrant.add_record(self.collection_names["documents_splitting"], point)

        return chunked_document

    def embed_chunked_document(self, chunked_document: ChunkedDocument):
        embedded_document = self.document_embedder.embed(chunked_document)
        document_chunks = chunked_document.chunks

        for embedded_chunk, document_chunk in zip(embedded_document.embedded_chunks, document_chunks):
            point_id = random.randint(1, 1000000)
            point: PointStruct = PointStruct(id=point_id, vector=embedded_chunk.embedding,
                                             payload=embedded_chunk.payload)
            with self.lock:
                self.qdrant.add_record(self.collection_names["documents_embedding"], point)

        return embedded_document




if __name__ == "__main__":
    # config = Config()
    # config["pipeline_stages"]["documents_parsing"]["label"] = "new_parse_label"
    # config["pipeline_stages"]["documents_splitting"]["label"] = "new_split_label"

    # Read environment variables for paths
    data_directory = os.getenv("DATA_DIR", "/app/data")
    config_path = os.getenv("CONFIG_PATH", "/app/conf/config.toml")

    orchestrator = UploadOrchestrator(data_directory, config_path)
    print("Starting document upload process...")
    orchestrator.upload()
    print("Document processing completed.")
